#!/usr/bin/python3
# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2016, 2018
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

from gettext import gettext as _
import re

pci_pattern = re.compile(r'\+\s*\d{4}:\d{2}:\d{2}\.\d')


class StorageDevicesException(Exception):
    def __init__(self, msg):
        super(StorageDevicesException, self).__init__(msg)
        self.msg = msg

    def __str__(self):
        return self.msg


class ExistingPartitionException(Exception):
    def __init__(self, msg):
        super(ExistingPartitionException, self).__init__(msg)
        self.msg = msg

    def __str__(self):
        return self.msg


class TargetDisk(object):
    '''
    This class represents the storage devices that will
    be discovered and used during the installation of the
    PVM partition.
    '''

    def __init__(self):
        # Array of lists, we will need to modify the "select field"
        # [(name, size, desc, vend, prod)]
        self._diskList = []
        # used to remove slave disks from _diskList
        self._mpathSlaveList = []
        # used to idenify slave disks of selected mpath
        self._mpathDict = {}
        # String of selected disk paths
        # This is the value in install config file
        # /dev/sdX OR /dev/mapper/mpathX
        self._selectedDisks = ""
        # Calculated value from file parsing
        # These are used in the
        self._padName = 0
        self._padSize = 0
        self._padDesc = 0
        # Cache error for reporting
        self._error = ""
        # Existing partition warning
        self._warningMsg = ""
        self._pciLocationDict = {}

    def buildDiskList(self, storagefile):
        '''
        Build the disk list and filter out redundant mpath disks
        :param storagefile: storage file object
        '''

        for line in storagefile:
            parsedLine = line.rstrip('\n').split('|')
            # MPATH disk
            if "mpath" in parsedLine[0]:
                # mpatha|sdg|sdc|sdf|559(GB)|pci
                name = parsedLine[0]
                size = parsedLine[-1]
                desc = "Multipath I/O Disk"
                slaves = parsedLine[1:-1]
                self._mpathSlaveList.extend(slaves)
                self._mpathDict[name] = slaves
                pci = parsedLine[-1]
                # fields not used for mpath
                vend = ""
                prod = ""
            else:
                # sde|SCSI Disk|IBM|2145|20480(MB)|pci
                (name, desc, vend, prod, size, pci) = parsedLine
                # convert Size MB -> GB
                # 61440(MB) -> 60(GB)
                # if "(MB)" in size:
                size_number = size.split("(MB)")[0]
                # Make sure its atleast a GB
                if (int(size_number) > 1024):
                    # No decimals
                    size = "{:.0f}{}".format((int(size_number)/1024), "(GB)")

            self._diskList.append([name, size, desc, vend, prod, pci])

    def removempathdevices(self):
        '''
        Filter out disks classified as multipath devices
        Get padding values for pretty columns
        4 columns: Name | Size | Desc | Vend+Prod
        '''
        disk_idx = 0
        while disk_idx < len(self._diskList):
            diskinfo = self.diskList[disk_idx]
            name = diskinfo[0]
            if name in self._mpathSlaveList:
                self._diskList.remove(diskinfo)
            else:
                size = diskinfo[1]
                desc = diskinfo[2]
                if(len(name) > self._padName):
                    self._padName = len(name)
                if(len(size) > self._padSize):
                    self._padSize = len(size)
                if(len(desc) > self._padDesc):
                    self._padDesc = len(desc)
                disk_idx += 1

    def parseFile(self):
        '''
        Parse file and catch I/O errors or parsing issues
        '''
        try:
            with open("/diskstorage.info", 'r') as storagefile:
                self.buildDiskList(storagefile)
                # remove mpath disks and gather padding
                self.removempathdevices()
                if not self._diskList:
                    raise StorageDevicesException(_("No storage devices were "
                                                    "found on the system"))
                else:
                    # set a default selection of first disk
                    # this will get overwritten when panel is created
                    # if user specified disk in grub cfg
                    self._selectedDisks = self._diskList[0][0]
        except IOError:
            raise StorageDevicesException(_("Unable to open file "
                                            "diskstorage.info"))

        try:
            with open("/location.info", 'r') as locationfile:
                for line in locationfile:
                    if pci_pattern.match(line):
                        split = line.split()
                        # Drop port information
                        split[2] = re.sub('-T.*$', '', split[2])
                        self._pciLocationDict.update({split[1]: split[2]})
        except IOError:
            raise StorageDevicesException(_("Unable to open file "
                                            "location.info"))

    @staticmethod
    def validate(selectedDisks):
        '''
        This method validates user selection
        No more than two disks can be selected
        An mpath disk cannot be selected with another disk
        '''
        # selectedDisks is a string, space seperated
        numdisks = len(selectedDisks.split())
        exmsg = ""
        if numdisks > 2:
            exmsg = _("Only up to two disks can be selected")
        elif numdisks > 1:
            for name in selectedDisks.split():
                if "mpath" in name:
                    exmsg = _("A multipath disk cannot be selected "
                              "with another disk")
        elif numdisks == 0:
            exmsg = _("At least one disk must be selected")

        if exmsg:
            raise StorageDevicesException(exmsg)
        else:
            return 0

    def checkPartitions(self, selectedDisks):
        '''
        Use fdisk -l output to determine if disk has existing partitions
        Show warning with list of disk name and partitions.
        '''
        try:
            with open("/partition.info", "r") as f:
                fdisk = f.readlines()

        except IOError:
            raise ExistingPartitionException(_("Unable to open file "
                                               "partitions.info"))
        # if no partitions found
        if not fdisk:
            return 0

        # First check to see if this is a mpath device
        # if so, check its slaves for partitions
        diskstocheck = []
        firstdisk = selectedDisks.split()[0]
        partition_str = ""
        mpath_name = ""
        new_disk_header = False
        show_warn = False
        if "mpath" in firstdisk:
            # save mpath name for later
            mpath_name = firstdisk.split("/")[-1]
            for slavedisk in self._mpathDict[mpath_name]:
                # check for slave disks partitions instead
                diskstocheck.append("/dev/{}".format(slavedisk))
        else:
            diskstocheck = selectedDisks.split()

        # loop through disk
        for disk_name in diskstocheck:
            new_disk_header = True
            for line in fdisk:
                # see if disk has a partition
                if disk_name in line:
                    show_warn = True
                    # grab sdb3 out of fdisk:
                    # /dev/sdb3  1015808 1172121599 1171105792 558.4G Linux LVM
                    partition_name = line.split()[0].split("/")[-1]
                    if new_disk_header:
                        # format info
                        # diskname:
                        #     partition, partition
                        partition_str = ("{}\n"
                                         "    {}:\n"
                                         "        {}").format(partition_str,
                                                              disk_name,
                                                              partition_name)
                        new_disk_header = False
                    else:
                        # add partitions to the line
                        partition_str = "{}, {}".format(partition_str,
                                                        partition_name)

        self._warningMsg = _("WARNING: All data on selected disk(s) will be "
                             "erased.\nThe following partitons will be "
                             "removed:\n{}\n\n"
                             "Do you want to continue?").format(partition_str)

        if show_warn:
            raise ExistingPartitionException(self._warningMsg)
        return 0

    @property
    def padName(self):
        return self._padName

    @property
    def padSize(self):
        return self._padSize

    @property
    def padDesc(self):
        return self._padDesc

    @property
    def warningMsg(self):
        return self._warningMsg

    @property
    def diskList(self):
        return self._diskList

    @property
    def selectedDisks(self):
        return self._selectedDisks

    @property
    def error(self):
        return self._error

    @property
    def pciLocationDict(self):
        return self._pciLocationDict

    @padName.setter
    def padName(self, padName):
        self._padName = padName

    @padSize.setter
    def padSize(self, padSize):
        self._padSize = padSize

    @padDesc.setter
    def padDesc(self, padDesc):
        self._padDesc = padDesc

    @warningMsg.setter
    def warningMsg(self, warningMsg):
        self._warnPartitions = warningMsg

    @diskList.setter
    def diskList(self, diskList):
        self._diskList = diskList

    @selectedDisks.setter
    def selectedDisks(self, selectedDisks):
        self._selectedDisks = selectedDisks

    @error.setter
    def error(self, error):
        self._error = error
