# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2017
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

import ast
from gettext import gettext as _
from snack import ButtonBar
from snack import Grid
from snack import GridForm
from snack import Label
from snack import SingleRadioButton
from snack import TextboxReflowed
from wizpkg.wizardpanel import WizardPanel
from pvmconfigdata import PVMConfigData as pcd


class SRIOVAdapterViewState(object):
    def __init__(self, end):
        self.start = 0
        self.end = end


class SRIOVModePanel(WizardPanel):

    '''
    This class extends the base WizardPanel and displays SRIOV Adapters,
    allowing users to choose the mode of those adapters (dedicated/Sriov).

      +---------------- SR-IOV Adapter Modes ------------------+
      |                                                        |
      |  Set the mode of SR-IOV adapters.                      |
      |                                                        |
      | Dedic.     Sriov     Location code   Description       |
      |  ( )        ( )      U79AB.xxxx      xxxxxx            |
      | .....                                                  |
      |                                                        |
      |  1 - 7 of 15                                           |
      |--------------------------------------------------------|
      | <view previous>  <view more>  <back>  <next>  <cancel> |
      +________________________________________________________+
    '''

    MAX_ENTRY = 7

    def __init__(self, screen, cfp, name_title, msg, sriov_adpts, view_state):
        name = name_title[0]
        paneltitle = name_title[1]
        gf = GridForm(screen, paneltitle, 1, 5)
        tb = TextboxReflowed(WizardPanel.WIDTH, msg)

        # Line #1: table header

        # Draw single table header line for all modes:
        # Dedicated
        # SRIOV
        mode_names = [_("Dedic."), _("SRIOV")]
        header_labels = mode_names.copy()
        header_labels.append(_("Location Code"))

        num_modes = len(mode_names)
        num_labels = len(header_labels)

        # Definitions of width of fields:
        # . The width of the wizard window is fixed as 72
        # . The width of the location code is fixed as 28
        # . The width of each radio button is fixed as 8
        _BOTTOM_DASH_LEN = 72
        _BUTTON_DASH_LEN = 8
        _SLOT_DASH_LEN = _BOTTOM_DASH_LEN - num_modes * _BUTTON_DASH_LEN
        maxrows = view_state.end - view_state.start
        g2 = Grid(num_labels, maxrows + 2)

        for (label_idx, label) in enumerate(header_labels):
            g2.setField(Label(label), label_idx, 0,
                        padding=(0, 0, 0, 0), anchorLeft=1)

        # Line #2: table header border (separator of header and table body)
        header_borders = ["-" * _BUTTON_DASH_LEN] * num_modes
        header_borders.append("-" * _SLOT_DASH_LEN)

        for (label_idx, border) in enumerate(header_borders):
            g2.setField(Label(border), label_idx, 1,
                        padding=(0, 0, 0, 0), anchorLeft=1)

        self.rbdicts = {}
        for sriov_mode in mode_names:
            self.rbdicts[sriov_mode] = {}
        sriov_mode_sect = cfp.get_attribute(cfp.SRIOV_ADAPTER_SECTION,
                                            cfp.SRIOV_MODE)
        adpts_in_sriov = []
        if sriov_mode_sect:
            adpts_in_sriov = ast.literal_eval(str(sriov_mode_sect))
        displayrow = 2

        rb0 = 0
        for sriov_adpt in sriov_adpts[view_state.start:view_state.end]:
            rb1 = SingleRadioButton(
                "", rb0, 0 if sriov_adpt in adpts_in_sriov else 1)
            g2.setField(rb1, 0, displayrow, padding=(0, 0, 0, 0), anchorLeft=1)
            self.rbdicts["Dedic."].update({sriov_adpt: rb1})
            rb2 = SingleRadioButton(
                "", rb1, 1 if sriov_adpt in adpts_in_sriov else 0)
            g2.setField(rb2, 1, displayrow, padding=(0, 0, 0, 0), anchorLeft=1)
            self.rbdicts["SRIOV"].update({sriov_adpt: rb2})
            g2.setField(Label(sriov_adpt), num_modes, displayrow,
                        padding=(0, 0, 1, 0), anchorLeft=1)
            displayrow += 1

        g3 = Grid(1, 2)
        g3.setField(Label(_("%d - %d of %d") % (view_state.start + 1,
                                                view_state.end,
                                                len(sriov_adpts))),
                    0, 0, padding=(0, 0, 0, 0), anchorLeft=1)
        g3.setField(Label("_" * _BOTTOM_DASH_LEN), 0, 1,
                    padding=(0, 0, 0, 0), anchorLeft=1)

        # Needed by super class, WizardPanel
        self.buttonBar = SRIOVModePanel.buildButtonBar(
            screen, len(sriov_adpts), view_state.start, view_state.end)

        gf.add(tb, 0, 0, padding=(0, 0, 1, 1), growx=1, anchorLeft=1)
        gf.add(g2, 0, 1, padding=(0, 1, 0, 0), growx=1)
        gf.add(g3, 0, 2, padding=(0, 1, 0, 0), growx=1)
        gf.add(self.buttonBar, 0, 3, padding=(0, 1, 0, 0), growx=1)
        super(SRIOVModePanel, self).__init__(screen=screen,
                                             name=name, gridform=gf,
                                             panelaction=None)

    @staticmethod
    def buildButtonBar(screen, max_rows, start, end):
        if max_rows <= SRIOVModePanel.MAX_ENTRY:
            iopbb = ButtonBar(screen, ((_("Back"), WizardPanel.BACK),
                                       (_("Next"), WizardPanel.NEXT),
                                       (_("Cancel"), WizardPanel.CANCEL)),
                              compact=1)
        elif start == 0:
            iopbb = ButtonBar(screen, ((_("View More"), WizardPanel.MORE),
                                       (_("Back"), WizardPanel.BACK),
                                       (_("Next"), WizardPanel.NEXT),
                                       (_("Cancel"), WizardPanel.CANCEL)),
                              compact=1)
        elif end < max_rows:
            iopbb = ButtonBar(screen, ((_("View Previous"), WizardPanel.PREV),
                                       (_("View More"), WizardPanel.MORE),
                                       (_("Back"), WizardPanel.BACK),
                                       (_("Next"), WizardPanel.NEXT),
                                       (_("Cancel"), WizardPanel.CANCEL)),
                              compact=1)
        else:
            iopbb = ButtonBar(screen, ((_("View Previous"), WizardPanel.PREV),
                                       (_("Back"), WizardPanel.BACK),
                                       (_("Next"), WizardPanel.NEXT),
                                       (_("Cancel"), WizardPanel.CANCEL)),
                              compact=1)
        return iopbb

    def _set_rbs(self, sriov_mode, rbdict, sriov_mode_dict):
        if str(sriov_mode) == "SRIOV":
            for drc, rb in rbdict.items():
                if rb.selected():
                    sriov_mode_dict["SRIOV"].append(drc)
        elif str(sriov_mode) == "Dedic.":
            for drc, rb in rbdict.items():
                if rb.selected():
                    sriov_mode_dict["Dedic."].append(drc)

    def getValues(self):
        '''
        Returns a list of adapter drcs to be set to SRIOV mode.
        '''
        sriov_mode_dict = {"SRIOV": [], "Dedic.": []}

        for sriov_mode, rbdict in self.rbdicts.items():
            self._set_rbs(sriov_mode, rbdict, sriov_mode_dict)

        return sriov_mode_dict


def _proc_prev_more(next_panel, sriov_view_state, sriov_adpts):
    MAX_ENTRY = SRIOVModePanel.MAX_ENTRY
    ret_val = False
    if next_panel == WizardPanel.PREV:
        sriov_view_state.start = sriov_view_state.start - MAX_ENTRY
        sriov_view_state.end = sriov_view_state.start + MAX_ENTRY
        ret_val = True
    elif next_panel == WizardPanel.MORE:
        sriov_view_state.start = sriov_view_state.start + MAX_ENTRY
        cond = (sriov_view_state.end + MAX_ENTRY > len(sriov_adpts))
        val = (sriov_view_state.end + MAX_ENTRY)
        sriov_view_state.end = len(sriov_adpts) if cond else val
        ret_val = True
    return ret_val


def _set_sriov(panel, cfp):
    s_vals = panel.getValues()
    sriov_sec = cfp.get_attribute(pcd.SRIOV_ADAPTER_SECTION,
                                  pcd.SRIOV_MODE)
    if sriov_sec:
        curr_sriovs = ast.literal_eval(str(sriov_sec))
        s_vals["SRIOV"].extend([cs for cs in curr_sriovs if (
            cs not in s_vals["Dedic."] and cs not in s_vals["SRIOV"])])

    cfp.set_attribute(pcd.SRIOV_ADAPTER_SECTION,
                      pcd.SRIOV_MODE, s_vals["SRIOV"])


def process_sriov_mode_panel(screen, cfp, sriov_adpts):
    view_end = len(sriov_adpts) if len(
        sriov_adpts) < SRIOVModePanel.MAX_ENTRY else SRIOVModePanel.MAX_ENTRY
    sriov_view_state = SRIOVAdapterViewState(view_end)
    while True:
        panel = SRIOVModePanel(
            screen, cfp, ("SRIOVModePanel", _("SR-IOV Adapter Modes")),
            _("Set the mode of SR-IOV adapters."), sriov_adpts,
            sriov_view_state)
        panel.run()
        nextpanel = panel.next()
        if nextpanel == WizardPanel.BACK:
            return WizardPanel.BACK
        elif nextpanel == WizardPanel.CANCEL:
            return WizardPanel.EXIT
        else:
            _set_sriov(panel, cfp)
            if not _proc_prev_more(nextpanel, sriov_view_state, sriov_adpts):
                return WizardPanel.NEXT


def read_sriov_lines():
    try:
        with open("/tmp/allsriovslots.cfg", "r") as sriovfile:
            sf = sriovfile.readlines()
            if sf:
                return [li.rstrip('\n').split(":")[0] for li in sf]
    except IOError:
        return []
    return []
