#!/usr/bin/python3
# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

from gettext import gettext as _
import xml.etree.ElementTree as xmlreader


class NoEthernetDevicesFoundException(Exception):
    def __init__(self, msg):
        super(NoEthernetDevicesFoundException, self).__init__(msg)
        self.msg = msg

    def __str__(self):
        return self.msg


class EthernetDevice(object):
    '''
    This class represents the ethernet devices that will
    be discovered and can be used during the installation of the
    PVM partition.
    '''

    def __init__(self):
        # Using array with tuples instead of dictionary
        # This provides predictable ordering
        self._available_ports = []
        # gets set to value in install-config file
        self._selected_port = ""
        self._padding = 0

    def findethernetdevices(self):
        # catch any IOerrors or parsing issues
        infofile = '/ethernet.info'
        try:
            self._available_ports = []
            ethlist = xmlreader.parse(infofile).getroot()
            for dev in ethlist.iter('node'):
                name = dev.find('logicalname').text
                if(len(name) > self._padding):
                    self._padding = len(name)

                mac = dev.find('serial').text
                location = self.findlocation(name)
                self._available_ports.append((name, mac, location))
                # find enabled interface
                if(dev.get('disabled') is None):
                    self._selected_port = name
        except:
            raise NoEthernetDevicesFoundException(_("Unable to discover "
                                                    "ethernet interfaces."
                                                    "\"%s\" not found "
                                                    "or invalid format."
                                                    % infofile))
        # if case where file is exists/parsable yet not devices were found
        if(len(self._available_ports) == 0):
            raise NoEthernetDevicesFoundException(_("No ethernet interfaces "
                                                    "found"))

    def findlocation(self, interface):
        lscfgfile = '/location.info'
        try:
            output = open(lscfgfile, 'r')
            for line in output:
                if interface in line:
                    location = line.split()
                    # ['+', 'enP1p80s0f3', 'U78C9.001.WZS0BJM-P1-C11-T4']
                    # return last section of line.
                    return location[2]

        except:
            raise NoEthernetDevicesFoundException(_("Error getting interface "
                                                    "location code."))

    @property
    def available_ports(self):
        return self._available_ports

    @property
    def selected_port(self):
        return self._selected_port

    @property
    def padding(self):
        return self._padding

    @available_ports.setter
    def available_ports(self, available_ports):
        self._available_ports = available_ports

    @selected_port.setter
    def selected_port(self, selected_port):
        self._selected_port = selected_port

    @padding.setter
    def padding(self, padding):
        self._padding = padding
