# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015, 2018
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

import ast
from gettext import gettext as _
from wizpkg.wizardpanel import WizardPanel
from pvmconfigdata import PVMConfigData as pcd
from networkbridge.networkbridgepanel import NetworkBridgePanel
from networkbridge.linkaggregationpanel import show_link_aggregation
import configparser


def _owner_to_pio(all_ports, bridge_names, adapter_assign_dict,
                  port_assign_dict):
    for line in all_ports:
        # Remove the port information from the location code
        # so that we can check the adapter name against the list selected
        # earlier.
        adpt_port = line.rstrip('\n').split('|')[0]
        pna = line.rstrip('\n').split('|')[0][::-1].split('-', 1)[1][::-1]
        for owner in bridge_names:
            pio_per_owner = adapter_assign_dict[owner]
            if len(pio_per_owner) > 0 and pna in pio_per_owner.keys():
                ports_per_bridge = port_assign_dict[owner]
                ports_per_bridge.update({adpt_port: pio_per_owner[pna]})
                port_assign_dict.update({owner: ports_per_bridge})
                break
            elif len(pio_per_owner) > 0 and adpt_port in pio_per_owner.keys():
                ports_per_bridge = port_assign_dict[owner]
                ports_per_bridge.update({adpt_port: pio_per_owner[adpt_port]})
                port_assign_dict.update({owner: ports_per_bridge})
                break


def run_network_bridge(screen, cfgparser, greenfield):
    '''
        Executes the Virtual Network Bridge Panels
    '''

    # 1. Retrieve the Network adapters that can be potentially used by each
    #    of the 4 Virtual Network Bridges -- VIOS1, VIOS2, SDN1, SDN2.
    #    Some of these Network Bridges may not have been enabled during the
    #    prior installation screens and hence may not have any physical
    #    adapters associated with them.
    # 2. Identify the ports in each Network adapter.  Create a dictionary of
    #    {<VirtualNetworkBridge> : <port_id>}
    # 3. Show the screen for selecting the port(s) and specifying the
    #    Link aggregation for each Virtual Network Bridge.

    bridge_names = ["SDN1", "SDN2", "VIOS1", "VIOS2", "SDE1", "SDE2"]
    # Configuration section names
    cfg_pio_sections = {}
    cfg_netwkbr_sections = {}
    for bridge in bridge_names:
        cfg_pio_sections.update({bridge: getattr(pcd, bridge +
                                                 '_PHYS_IO_CONFIG_SECTION')})
        cfg_netwkbr_sections.update({bridge: getattr(pcd, bridge +
                                                     '_NETWORK_BRIDGE')})

    # Create a dictionary of boolean values to identify whether
    # each Network bridge is enabled or not.
    installmode = cfgparser.get_attribute(pcd.PVM_CONFIG_SECTION,
                                          pcd.INSTALLMODE)
    # Set viosenabled to False if this is non greenfield installation
    if (not greenfield):
        viosenabled = False
    else:
        viosenabled = (installmode == pcd.SDN or installmode == "")
        # Older config files do not have the InstallMode, so handle that
        #  special case here:  if the InstallMode option itself does not
        #  exist, then assume that the operator is using a old config file
        #  and expects VIOS to be installed.
        if (not cfgparser.config.has_option(pcd.PVM_CONFIG_SECTION,
                                            pcd.INSTALLMODE)):
            viosenabled = True

    ioredundant = cfgparser.get_attribute_boolean(pcd.PVM_CONFIG_SECTION,
                                                  pcd.IOREDUNDANCY, False)
    sdnenabled = cfgparser.get_attribute_boolean(pcd.SDN_CONFIG_SECTION,
                                                 pcd.SDN_ENABLED, False)

    sde_enabled = installmode == pcd.SDE
    sdnredundant = cfgparser.get_attribute_boolean(pcd.SDN_CONFIG_SECTION,
                                                   pcd.SDN_REDUN_ENABLED,
                                                   False)
    netwkbr_is_enabled = {"SDN1": sdnenabled and not sde_enabled,
                          "SDN2": sdnenabled and sdnredundant,
                          "VIOS1": viosenabled,
                          "VIOS2": viosenabled and ioredundant,
                          "SDE1": sde_enabled,
                          "SDE2": sde_enabled and ioredundant}

    # adapter_assign_dict is a dictionary of dictionaries with the layout
    #     as given below:
    # Key: one of the 4 values ("SDN1", "SDN2", "VIOS1", "VIOS2")
    # Value:  A dictionary of multiple {adapter-location: adapter-description}
    #          entries.
    adapter_assign_dict = {}
    # port_assign_dict is a dictionary of dictionaries with the layout
    #     as given below:
    # Key: one of the 4 values ("SDN1", "SDN2", "VIOS1", "VIOS2")
    # Value:  A dictionary of multiple {port-location: adapter-description}
    #          entries.
    port_assign_dict = {}

    # 1. Retrieve the Network adapters that can be potentially used by each
    #    of the 4 Virtual Network Bridges -- VIOS1, VIOS2, SDN1, SDN2.
    #    Read the configuration file and construct adapter_assign_dict
    #     dictionary:
    for owner, cfg_section in cfg_pio_sections.items():
        cfg_pio_per_owner = cfgparser.get_attribute(cfg_section, pcd.INFO)
        pio_per_owner = ({} if cfg_pio_per_owner == ''
                         else ast.literal_eval(str(cfg_pio_per_owner)))
        adapter_assign_dict.update({owner: pio_per_owner})
        port_assign_dict[owner] = {}

    #  2. Identify the ports in each Network adapter. Read the
    #      /networkports.info file to identify the list of ports available
    #      in each Network adapter and construct port_assign_dict dictionary.
    #  Sample data in the /networkports.info file
    #    U78CD.001.FZH8716-P2-C5-T1 | '1 Gigabit Ethernet (UTP) 4 ...'
    #    U78CD.001.FZH8716-P1-C5-T1 | '8 Gigabit PCI Express Dual ...'
    #    U78CD.001.FZH8716-P2-C2-T1 | 'PCIe2 2-Port 10GbE Base-T Adapter'
    with open("/networkports.info", 'r') as selectedfile:
        # There are 2 cases here: A port may be explicitly selected as a
        # "physical adapter" in the case of SRIOV ports being assigned
        # across owners, or the traditional case of just the adapter
        # being selected.
        all_ports = [line for line in selectedfile]
        _owner_to_pio(all_ports, bridge_names, adapter_assign_dict,
                      port_assign_dict)

    # If a Network bridge is enabled but no IO slots are assigned to it, then
    #  report an error.
    # Possibly a prior incomplete install where io slots were removed
    # and user hasn't done a reset
    # If this condition is detected, show the error message but allow
    # user to stay in wizard and go back to repair to do reset
    errmsg = ("Unable to detect any network ports for %s. If IO slots were "
              "removed from the NovaLink partition on a previous failed "
              "installation, you must reset the system")
    for bridge in bridge_names:
        if (netwkbr_is_enabled[bridge]):
            error_cond = False
            try:
                num_ports = len(port_assign_dict[bridge])
            except KeyError:
                error_cond = True
            if (num_ports <= 0):
                error_cond = True
            if (error_cond):
                WizardPanel.showError(screen, _(errmsg % bridge))
                return (WizardPanel.BACK)

    # 3. Show the screens for selecting the port(s) for each Virtual Network
    #    Bridge and specifying the Link aggregation for each type of Virtual
    #    Network Bridge.

    # pvid_display controls whether the Port VLAN ID should be shown in the
    #  Virtual Network Bridge panel.
    pvid_display = {"SDN1": False, "SDN2": False, "VIOS1": True,
                    "VIOS2": True, "SDE1": False, "SDE2": False}

    # pvid_edit controls whether the Port VLAN ID can be edited when shown
    #   in the Virtual Network Bridge panel.
    pvid_edit = {"SDN1": False, "SDN2": False, "VIOS1": True,
                 "VIOS2": False, "SDE1": False, "SDE2": False}

    # panel_seq[] contains the sequence of the panels to be shown as part of
    #  the Virtual Network Bridge selection.
    panel_seq = []
    # Build up the panel_seq list based on which bridge is enabled.
    if (netwkbr_is_enabled["SDN1"]):
        panel_seq.append("SDN1_PORTS")
        if (netwkbr_is_enabled["SDN2"]):
            panel_seq.append("SDN2_PORTS")
        panel_seq.append("SDN_LINKAGGR")
    if (netwkbr_is_enabled["VIOS1"]):
        panel_seq.append("VIOS1_PORTS")
        if (netwkbr_is_enabled["VIOS2"]):
            panel_seq.append("VIOS2_PORTS")
        panel_seq.append("VIOS_LINKAGGR")
    if (netwkbr_is_enabled["SDE1"]):
        panel_seq.append("SDE1_PORTS")
        if (netwkbr_is_enabled["SDE2"]):
            panel_seq.append("SDE2_PORTS")

    cur_panel = ''
    i = 0
    saved_pvid = ''
    if (len(panel_seq) <= 0):
        return (WizardPanel.NEXT)

    while True:
        cur_panel = panel_seq[i]
        if ("_PORTS" in cur_panel):
            # Convert the panel name  to the bridge name by removing the
            #  "_PORTS" suffix.
            bridge = cur_panel.replace('_PORTS', '')
            ports_per_bridge = port_assign_dict[bridge]

            # Retrieve the default/saved values from the configuration file.
            saved_ports = cfgparser.get_attribute(cfg_netwkbr_sections[bridge],
                                                  pcd.INFO)
            # Converting retrieved values to list
            saved_ports = ([] if saved_ports == ''
                           else ast.literal_eval(str(saved_ports)))

            # For SDN1 and SDN2, pvid will be set to an empty string ('').
            try:
                pvid = cfgparser.get_attribute(cfg_netwkbr_sections[bridge],
                                               pcd.PVID)
            except configparser.Error:
                pvid = ''
            # For VIOS2, used the saved PVID from previous VIOS1
            # set in the next panel
            if bridge == 'VIOS2':
                pvid = saved_pvid

            bridge_info = {}
            nbp = NetworkBridgePanel
            bridge_info[nbp.BRIDGE_NAME] = bridge
            bridge_info[nbp.BRIDGE_PVID] = pvid
            bridge_info[nbp.BRIDGE_PVID_SHOW] = pvid_display[bridge]
            bridge_info[nbp.BRIDGE_PVID_EDIT] = pvid_edit[bridge]
            panel = NetworkBridgePanel(ports_per_bridge, saved_ports,
                                       bridge_info)
            (nextpanel, ports_per_bridge, pvid) =\
                panel.run_networkbridge(screen)

            # Save the selected values to the configuration file
            bridge = bridge.replace('SDE', 'SDN')
            cfgparser.set_attribute(cfg_netwkbr_sections[bridge], pcd.INFO,
                                    ports_per_bridge)
            cfgparser.set_attribute(cfg_netwkbr_sections[bridge], pcd.PVID,
                                    pvid)
            # Once the PVID for VIOS1 is saved into the file, save it
            # in saved_pvid so the next VIOS2 panel can use it
            if bridge == 'VIOS1':
                saved_pvid = pvid
        elif ("_LINKAGGR" in cur_panel):
            (rv1, rv2) = _check_num_ports(cur_panel, cfgparser,
                                          cfg_netwkbr_sections,
                                          netwkbr_is_enabled)
            need_link_aggr = rv1
            linkaggr_bridge = rv2
            if (need_link_aggr):
                nextpanel = show_link_aggregation(screen, cfgparser,
                                                  linkaggr_bridge)

        if nextpanel == WizardPanel.NEXT:
            i += 1
        elif nextpanel == WizardPanel.CANCEL:
            return WizardPanel.EXIT
        elif nextpanel == WizardPanel.BACK:
            i -= 1

        if (i < 0):
            return nextpanel
        elif (i >= len(panel_seq)):
            return (WizardPanel.NEXT)


def _check_num_ports(cur_panel, cfgparser, cfg_netwkbr_sections,
                     netwkbr_is_enabled):
    need_link_aggr = False
    linkaggr_bridge = ''
    if (cur_panel == "SDN_LINKAGGR"):
        linkaggr_bridge = "SDN"
        bridge1 = "SDN1"
        bridge2 = "SDN2"
    elif (cur_panel == "VIOS_LINKAGGR"):
        linkaggr_bridge = "VIOS"
        bridge1 = "VIOS1"
        bridge2 = "VIOS2"

    portlist1 = cfgparser.get_attribute(cfg_netwkbr_sections[bridge1],
                                        pcd.INFO)
    portlist1 = ([] if portlist1 == '' else ast.literal_eval(str(portlist1)))
    portlist2 = cfgparser.get_attribute(cfg_netwkbr_sections[bridge2],
                                        pcd.INFO)
    portlist2 = ([] if portlist2 == '' else ast.literal_eval(str(portlist2)))

    if (netwkbr_is_enabled[bridge1] and (len(portlist1) > 1)):
        if (netwkbr_is_enabled[bridge2]):
            if (len(portlist2) > 1):
                need_link_aggr = True
            else:
                need_link_aggr = False
        else:
            need_link_aggr = True

    return (need_link_aggr, linkaggr_bridge)
