# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

from gettext import gettext as _
from snack import Entry
from snack import Grid
from snack import GridForm
from snack import Label
from snack import TextboxReflowed
from wizpkg.wizardpanel import WizardPanel
from wizpkg.extendedwizardpanels import buildButtonBar
from user.user import User
from user.user import BadUserNameException


class UserPasswordWizardPanel(WizardPanel):
    '''
    This class extends the base WizardPanel and uses
    one or more Entry as widget in its gridform
    User will be allowed to enter values into the entry
    fields.
    Password fields will be hidden.
             +-------------------------------+
             |     some title                |
             +-------------------------------+
             | Here are some instructions..  |
             |                               |
             | User name        ____________ |
             | Password         ____________ |
             | Reenter Password ____________ |
             | Enter password for padmin     |
             | Password         ____________ |
             | Reenter Password ____________ |
             |                               |
             | <back>   <next>   <cancel>    |
             +-------------------------------+
    '''

    def __init__(self, screen, name, paneltitle, msg, username,
                 passwd1, passwd2, passwd3, passwd4,
                 show_vios, index, panelaction):

        gf = GridForm(screen, paneltitle, 1, 3)
        tb = TextboxReflowed(WizardPanel.WIDTH, msg)
        # Save entry in separate Grid and get a handle to it
        # to clear data when transitioning to previous panel
        g = Grid(2, 6)
        g.setField(Label(_("NovaLink's user name ")), 0, 0,
                   padding=(2, 0, 1, 0), anchorLeft=1)
        self.username = Entry(30, text=username)
        self.show_vios = show_vios
        g.setField(self.username, 1, 0, anchorLeft=1)
        g.setField(Label(_("Password ")), 0, 1, padding=(2, 0, 1, 0),
                   anchorLeft=1)
        self.pvmpasswd = Entry(30, text=passwd1, password=1)
        g.setField(self.pvmpasswd, 1, 1, padding=(0, 0, 1, 0), anchorLeft=1)

        g.setField(Label(_("Reenter Password ")), 0, 2, padding=(2, 0, 1, 0),
                   anchorLeft=1)
        self.pvmpasswd2 = Entry(30, text=passwd2, password=1)
        g.setField(self.pvmpasswd2, 1, 2, padding=(0, 0, 1, 0), anchorLeft=1)

        self.viospasswd = Entry(30, text=passwd3, password=1)
        self.viospasswd2 = Entry(30, text=passwd4, password=1)
        if show_vios:
            vtb = TextboxReflowed(40,
                                  _("Enter password for VIOS padmin user"))
            g.setField(vtb, 0, 3, padding=(0, 1, 1, 0), anchorLeft=1)
            g.setField(Label(_("Password ")), 0, 4, padding=(2, 1, 1, 0),
                       anchorLeft=1)
            g.setField(self.viospasswd, 1, 4, padding=(0, 1, 1, 0),
                       anchorLeft=1)
            g.setField(Label(_("Reenter Password ")), 0, 5,
                       padding=(2, 0, 1, 0), anchorLeft=1)
            g.setField(self.viospasswd2, 1, 5, padding=(0, 0, 1, 0),
                       anchorLeft=1)

        # Adding widgets to GridForm. First and last widgets
        # will have padding at bottom and top respectively
        # to provide nice spacing.
        gf.add(tb, 0, 0, padding=(0, 1, 1, 1), anchorLeft=1)
        gf.add(g, 0, 1, growy=1, anchorTop=0)
        bb = buildButtonBar(screen, index)
        gf.add(bb, 0, 2, padding=(0, 1, 0, 0), growx=1)
        self.buttonBar = bb
        super(UserPasswordWizardPanel, self).__init__(screen=screen,
                                                      name=name, gridform=gf,
                                                      panelaction=panelaction)

    def getValues(self):
        '''
        Obtain the entry list. This will be a list
        of tuple. Caller needs to iterate thru the
        list, then obtain the 2nd tuple of each entry
        '''
        entryList = []
        entryList.append(self.username.value())
        entryList.append(self.pvmpasswd.value())
        entryList.append(self.pvmpasswd2.value())
        entryList.append(self.viospasswd.value())
        entryList.append(self.viospasswd2.value())
        return entryList

    def validate(self):
        '''
        This method validate user input on this wizard panel. Things checked:
        - The 2 entered passwords does not match
        - May want to tie in strong password policy
        '''
        rc = 0
        try:
            User.validateusername(self.username.value())
        except BadUserNameException as e:
            self.showError(self.screen, str(e))
            rc = 1
        if self.show_vios:
            if "" in [self.viospasswd.value(), self.viospasswd2.value(),
                      self.pvmpasswd.value(), self.pvmpasswd2.value()]:
                self.showError(self.screen, _("Password cannot be empty"))
                rc = 1
            if self.viospasswd.value() != self.viospasswd2.value():
                self.showError(self.screen, _("Password entered for VIOS "
                                              "user do not match"))
                rc = 1
        else:
            if "" in [self.pvmpasswd.value(), self.pvmpasswd2.value()]:
                self.showError(self.screen, _("Password cannot be empty"))
                rc = 1
        if self.pvmpasswd.value() != self.pvmpasswd2.value():
            self.showError(self.screen, _("Password entered for PVM "
                                          "user do not match"))
            rc = 1

        return rc
