# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

from snack import FLAG_DISABLED
from snack import FLAGS_SET
from snack import snackArgs
from snack import ButtonChoiceWindow
from snack import Checkbox
from snack import CheckboxTree
from snack import Entry
from snack import Grid
from snack import GridForm
from snack import Label
from snack import Listbox
from snack import RadioBar
from snack import SingleRadioButton
from snack import Textbox
from snack import ButtonBar
from snack import TextboxReflowed
from gettext import gettext as _
from wizpkg.wizardpanel import WizardPanel
from math import ceil

'''
Exception used by this class
'''


class MissingTupleException(Exception):

    def __init__(self, arg):
        self.msg = arg

    def __str__(self):
        return self.msg


def buildButtonBar(screen, position):
    '''
    Convenience method to build radio button bar
    Only handles back, next, cancel or next, cancel or back, finish, cancel
    :param screen: SnackScreen instance
    :param position: Panel position dictating how to build button bar
    '''
    if position == WizardPanel.FIRST:
        bb = ButtonBar(screen, ((_("Next"), WizardPanel.NEXT),
                                (_("Cancel"), WizardPanel.CANCEL)), compact=1)
    elif position == WizardPanel.LAST:
        bb = ButtonBar(screen, ((_("Back"), WizardPanel.BACK),
                                (_("Finish"), WizardPanel.FINISH),
                                (_("Cancel"), WizardPanel.CANCEL)), compact=1)
    else:
        bb = ButtonBar(screen, ((_("Back"), WizardPanel.BACK),
                                (_("Next"), WizardPanel.NEXT),
                                (_("Cancel"), WizardPanel.CANCEL)), compact=1)
    return bb


class ListBoxWizardPanel(WizardPanel):
    '''
    This class extends the base WizardPanel and uses
    Listbox as widget in its gridform

             +-------------------------------+
             |     some title                |
             +-------------------------------+
             | Here are some instructions..  |
             |                               |
             | selectable entry 1            |
             | selectable entry 2            |
             | ...                           |
             |                               |
             | <back>   <next>   <cancel>    |
             +-------------------------------+
    '''

    def __init__(self, screen, name, paneltitle, msg, entrylist,
                 position, panelaction):
        '''
        Init method
        :param screen: SnackScreen instance
        :param name: Name of the panel
        :param paneltitle: Panel title displayed to user
        :param msg: Message displayed to user as instructions
        :param entrylist is a list of tuples where each tuple represents
               the value to be displayed and the returned value selected
               (single selection)
        :param position: Panel position in the wizard
        :param panelaction: Function to called when user transition out
        '''
        height = len(entrylist)
        gf = GridForm(screen, paneltitle, 1, height+2)
        tb = Textbox(WizardPanel.WIDTH, 1, msg, scroll=0, wrap=1)
        lb = Listbox(height, scroll=1, width=WizardPanel.WIDTH)
        for item in entrylist:
            if type(item) == tuple:
                (text, val) = item
            else:
                raise MissingTupleException(_("Invalid tuple passed to "
                                              "ListBoxWizardPanel"))
            lb.append(text, val)

        self.listbox = lb
        # Adding widgets to GridForm. First and last widgets
        # will have padding at bottom and top respectively
        # to provide nice spacing.
        gf.add(tb, 0, 0, padding=(0, 0, 0, 1), anchorTop=0)
        gf.add(lb, 0, 1, growy=1, anchorTop=0)
        bb = buildButtonBar(screen, position)
        gf.add(bb, 0, 3, padding=(0, 1, 0, 0), growx=1)
        self.buttonBar = bb
        super(ListBoxWizardPanel, self).__init__(screen=screen,
                                                 name=name, gridform=gf,
                                                 panelaction=panelaction)

    def getValues(self):
        '''
        Obtain values selected
        This returns 1 key that represent the current selection
        '''
        return self.listbox.current()


class ListEntryWizardPanel(WizardPanel):
    '''
    This class extends the base WizardPanel and uses
    one or more Entry as widget in its gridform
    User will be allowed to enter values into the entry
    fields.
             +-------------------------------+
             |     some title                |
             +-------------------------------+
             | Here are some instructions..  |
             |                               |
             | User entry 1 ______________   |
             | User entry 2 ______________   |
             | ...                           |
             |                               |
             | <back>   <next>   <cancel>    |
             +-------------------------------+
    '''

    def __init__(self, screen, name, paneltitle, msg, labellist,
                 position, panelaction):
        '''
        Init method
        :param screen: SnackScreen instance
        :param name: Name of the panel
        :param paneltitle: Panel title displayed to user
        :param msg: Message displayed to user as instructions
        :param labellist: List of strings representing the label
               to be displayed for each entry.
        :param position: Panel position in wizard
        :param panelaction: Function to called when user transition out
        '''
        height = len(labellist)
        gf = GridForm(screen, paneltitle, 1, height+2)
        tb = Textbox(WizardPanel.WIDTH, 1, msg, scroll=0, wrap=1)

        # Save entry in separate Grid and get a handle to it
        # to clear data when transitioning to previous panel
        count = 0
        self.entryList = []
        g = Grid(2, height)
        for l in labellist:
            e = Entry(40)
            g.setField(Label(l), 0, count, padding=(0, 0, 1, 0), anchorLeft=1)
            g.setField(e, 1, count, anchorLeft=1)
            count = count + 1
            self.entryList.append(e)
        self.cbgrid = g
        # Adding widgets to GridForm. First and last widgets
        # will have padding at bottom and top respectively
        # to provide nice spacing.
        gf.add(tb, 0, 0, padding=(0, 0, 0, 1), anchorTop=0)
        gf.add(g, 0, 1, growy=1, anchorTop=0)
        bb = buildButtonBar(screen, position)
        gf.add(bb, 0, 2, padding=(0, 1, 0, 0), growx=1)
        self.buttonBar = bb
        super(ListEntryWizardPanel, self).__init__(
            screen=screen, name=name, gridform=gf, panelaction=panelaction)

    def getValues(self):
        '''
        Obtain the entry list. This will be a list
        of tuple. Caller needs to iterate thru the
        list, then obtain the 2nd tuple of each entry
        '''
        return self.entryList

    def showerrors(self):
        '''
        Show pop up windows when encountering data errors during validation.
        '''
        ButtonChoiceWindow(self.screen, _("Error"),
                           _("Validation of data on panel failed.\
                           Please check your input data."),
                           buttons=[_("OK")])


class RadioButton2ColumnWizardPanel(WizardPanel):
    '''
    This class extends the base WizardPanel and uses
    2 columns of one or more RadioButton as widget in its gridform.
             +----------------------------------------------------------+
             |                      some title                          |
             +----------------------------------------------------------+
             |  Instructions 1                  Instructions 2          |
             |                                                          |
             | [ ] Some text1                  [ ] text1                |
             | [ ] Some other text1            [ ] Some other text2     |
             | ...                             ...                      |
             |                                                          |
             |             <back>   <next>   <cancel>                   |
             +----------------------------------------------------------+
    '''
    def __init__(self, screen, panelmsgs, radiobuttonlist1, radiobuttonlist2,
                 position, panelaction):
        '''
        Init method
        :param screen: SnackScreen instance
        :param panelmsgs[0]: Name of the panel
        :param panelmsgs[1]: Panel title displayed to user
        :param panelmsgs[2]: Common Instructions for both the columns
        :param panelmsgs[3]: Instructions for the Left column radio buttons
        :param panelmsgs[4]: Instructions for the right column radio buttons
        :param radiobuttonlist1: List of tuple of 3 entries: text, value,
               selected where text is the text to be displayed by
               the radio button, value is the value associated with
               the button used when returning current selection, and
               selected is the indication of whether the button should be
               selected.
        :param radiobuttonlist2: same format as radiobuttonlist1.
        :param position: Panel position in wizard
        :param panelaction: Function to be called when user transitions out
        '''
        name = panelmsgs[0]
        paneltitle = panelmsgs[1]
        msg = panelmsgs[2]
        msg1 = panelmsgs[3]
        msg2 = panelmsgs[4]
        rbheight = max(len(radiobuttonlist1), len(radiobuttonlist2))
        # The 'msg' text will be displayed in 2 adjacent columns.  So, some
        # slicing of the text is needed.
        slicelen = int(WizardPanel.WIDTH / 2)
        num_slices = ceil(len(msg) / slicelen)
        tbheight = int((num_slices + 1) / 2)
        tb_list = []
        for i in range(num_slices):
            tb_list.append(Textbox(int(WizardPanel.WIDTH / 2), 1,
                                   msg[(i*slicelen):((i+1)*slicelen)],
                                   scroll=0, wrap=0))
#        tb = Textbox(int(WizardPanel.WIDTH / 2), 2, msg, scroll=0, wrap=1)
        tb1 = Textbox(int(WizardPanel.WIDTH / 2), 1, msg1, scroll=0, wrap=1)
        tb2 = Textbox(int(WizardPanel.WIDTH / 2), 1, msg2, scroll=0, wrap=1)

        for item in (radiobuttonlist1 + radiobuttonlist2):
            if type(item) != tuple:
                raise MissingTupleException(_("Invalid tuple passed to "
                                              "RadioButtonWizardPanel"))

        rb1 = RadioBar(screen, radiobuttonlist1)
        self.radiobar1 = rb1
        rb2 = RadioBar(screen, radiobuttonlist2)
        self.radiobar2 = rb2
        gf = GridForm(screen, paneltitle, 2, rbheight+tbheight+2)
        for i in range(num_slices):
            gf.add(tb_list[i], (i % 2), int(i / 2), padding=(0, 0, 0, 0),
                   anchorLeft=True)
        cur_row = tbheight-1
        gf.add(tb1, 0, cur_row+1, padding=(0, 1, 0, 1), anchorLeft=True)
        gf.add(tb2, 1, cur_row+1, padding=(0, 1, 0, 1), anchorLeft=True)
        gf.add(rb1, 0, cur_row+2, growy=1, anchorLeft=True)
        gf.add(rb2, 1, cur_row+2, growy=1, anchorLeft=True)
        bb = buildButtonBar(screen, position)
        gf.add(bb, 0, 4, padding=(0, 1, 0, 0), growx=1)
        self.buttonBar = bb
        super(RadioButton2ColumnWizardPanel, self).__init__(
            screen=screen, name=name, gridform=gf, panelaction=panelaction)

    def getValues(self):
        '''
        Obtain values selected
        This will be the key associated with the selected radio button.
        '''
        return ([self.radiobar1.getSelection(),
                 self.radiobar2.getSelection()])


class RadioButtonWizardPanel(WizardPanel):
    '''
    This class extends the base WizardPanel and uses
    one or more RadioButton as widget in its gridform.
             +-------------------------------+
             |     some title                |
             +-------------------------------+
             | Here are some instructions..  |
             |                               |
             | [ ] Some text                 |
             | [ ] Some other text           |
             | ...                           |
             |                               |
             | <back>   <next>   <cancel>    |
             +-------------------------------+
    '''
    def __init__(self, screen, name, paneltitle, msg, radiobuttonlist,
                 position, panelaction):
        '''
        Init method
        :param screen: SnackScreen instance
        :param name: Name of the panel
        :param paneltitle: Panel title displayed to user
        :param msg: Message displayed to user as instructions
        :param radiobuttonlist: List of tuple of 3 entries: text, value,
               selected where text is the text to be displayed by
               the radio button, value is the value associated with
               the button used when returning current selection, and
               selected is the indication of whether the button should be
               selected.
        :param position: Panel position in wizard
        :param panelaction: Function to called when user transition out
        '''
        height = len(radiobuttonlist)
        gf = GridForm(screen, paneltitle, 1, height+2)
        tb = Textbox(WizardPanel.WIDTH, 1, msg, scroll=0, wrap=1)

        for item in radiobuttonlist:
            if type(item) == tuple:
                pass
            else:
                raise MissingTupleException(_("Invalid tuple passed to "
                                              "RadioButtonWizardPanel"))

        rb = RadioBar(screen, radiobuttonlist)
        self.radiobar = rb
        gf.add(tb, 0, 0, padding=(0, 0, 0, 1), anchorTop=0)
        gf.add(rb, 0, 1, growy=1, anchorTop=0)
        bb = buildButtonBar(screen, position)
        gf.add(bb, 0, 3, padding=(0, 1, 0, 0), growx=1)
        self.buttonBar = bb
        super(RadioButtonWizardPanel, self).__init__(
            screen=screen, name=name, gridform=gf, panelaction=panelaction)

    def getValues(self):
        '''
        Obtain values selected
        This will be the key associated with the selected radio button.
        '''
        return self.radiobar.getSelection()


class RadioButtonDescriptionWizardPanel(WizardPanel):
    '''
    This class extends the base WizardPanel and uses
    one or more RadioButton with a description as widget in its gridform.
             +-------------------------------+
             |     some title                |
             +-------------------------------+
             | Here are some instructions..  |
             |                               |
             | [ ] Some text                 |
             |     text Description          |
             | [ ] Some other text           |
             |     test Description          |
             | ...                           |
             |                               |
             | <back>   <next>   <cancel>    |
             +-------------------------------+
    '''
    def __init__(self, screen, name, paneltitle, msg, radiobuttonlist,
                 textlist, position, panelaction):
        '''
        Init method
        :param screen: SnackScreen instance
        :param name: Name of the panel
        :param paneltitle: Panel title displayed to user
        :param msg: Message displayed to user as instructions
        :param radiobuttonlist: List of tuple of 3 entries: text, value,
               selected where text is the text to be displayed by
               the radio button, value is the value associated with
               the button used when returning current selection, and
               selected is the indication of whether the button should be
               selected.
        :param textlist: A tuple of two entries: text description, line number.
                The text description is the text under the radiobutton title,
                and the line number value is the number of newlines the text
                contains.
        :param position: Panel position in wizard
        :param panelaction: Function to called when user transition out
        '''

        self.prevbutton = None
        self.buttonlist = []

        height = 0
        # Tuple format checking

        for item in textlist:
            if type(item) == tuple:
                height += item[1]
            else:
                raise MissingTupleException(_("Invalid text list tuple passed\
                                              to RadioButtonWizardPanel"))
        for item in radiobuttonlist:
            if type(item) == tuple:
                pass
            else:
                raise MissingTupleException(_("Invalid radiobuttonlist "
                                              "tuple passed to "
                                              "RadioButtonWizardPanel"))

        height += len(radiobuttonlist)
        gf = GridForm(screen, paneltitle, 1, height+2)
        tb = TextboxReflowed(WizardPanel.WIDTH, msg)
        g = Grid(1, (len(radiobuttonlist)+len(textlist)))
        x = 0
        # Iterate through buttons and corresponding description

        for (button, text) in zip(radiobuttonlist, textlist):
            (btext, bvalue, bselected) = button
            ttext = text[0]
            # Save button

            self.nextbutton = SingleRadioButton(btext,
                                                self.prevbutton, bselected)
            self.prevbutton = self.nextbutton
            self.buttonlist.append((self.nextbutton, bvalue))
            # display button

            g.setField(
                self.nextbutton, 0, x, padding=(0, 0, 1, 0), anchorLeft=1)
            x += 1
            g.setField(TextboxReflowed(WizardPanel.WIDTH, ttext),
                       0, x, padding=(4, 0, 1, 1), anchorLeft=1)
            x += 1
        # Add buttons and text to the panel

        gf.add(tb, 0, 0, padding=(0, 0, 0, 1), anchorTop=0)
        gf.add(g, 0, 1, growy=1, anchorTop=0)
        bb = buildButtonBar(screen, position)
        gf.add(bb, 0, 3, padding=(0, 1, 0, 0), growx=1)
        self.buttonBar = bb
        super(RadioButtonDescriptionWizardPanel, self).__init__(
            screen=screen, name=name, gridform=gf, panelaction=panelaction)

    def getValues(self):
        '''
        Obtain values selected
        This will be the key associated with the selected radio button.
        '''
        for (b, value) in self.buttonlist:
            if b.selected():
                return value

        return None


class CheckboxTreeWizardPanel(WizardPanel):
    '''
    This class extends the base WizardPanel and uses
    CheckboxTree as widget in its gridform.

             +-------------------------------+
             |     some title                |
             +-------------------------------+
             | Here are some instructions..  |
             |                               |
             | + Parent node                 |
             |   [ ] Child node              |
             | ...                           |
             |                               |
             | <back>   <next>   <cancel>    |
             +-------------------------------+
    There can be multiple root nodes with this wizard panel.
    '''
    def __init__(self, screen, name, paneltitle, msg, childentrylist,
                 position, panelaction):
        '''
        Init method
        :param screen: SnackScreen instance
        :param name: Name of the panel used as the name of the root node.
        :param paneltitle: Panel title displayed to user
        :param msg: Message displayed to user as instructions
        :param childentrylist: Tuple list of 2 entries: text, value
               where text is the text to be displayed by the checkbox,
               value is the value associated with the checkbox when
               returning current selection
        :param position: Panel position in wizard
        :param panelaction: Function to called when user transition out
        '''
        height = len(childentrylist)
        gf = GridForm(screen, paneltitle, 1, 3)
        tb = Textbox(WizardPanel.WIDTH, 1, msg, scroll=0, wrap=1)
        ct = CheckboxTree(height=height, scroll=1)
        ct.append(name)
        for item in childentrylist:
            if type(item) == tuple:
                (text, val) = item
                ct.addItem(text, (0, snackArgs['append']), val)
            else:
                raise MissingTupleException(_("Invalid tuple passed to "
                                              "CheckBoxTreeWizardPanel"))
        self.checkboxtree = ct
        self.entrylist = childentrylist
        # Adding widgets to GridForm. First and last widgets
        # will have padding at bottom and top respectively
        # to provide nice spacing.
        gf.add(tb, 0, 0, padding=(0, 0, 0, 1), anchorTop=0)
        gf.add(ct, 0, 1, growy=1, anchorLeft=1)
        bb = buildButtonBar(screen, position)
        gf.add(bb, 0, 2, padding=(0, 1, 0, 0), growx=1)
        self.buttonBar = bb
        super(CheckboxTreeWizardPanel, self).__init__(screen=screen,
                                                      name=name, gridform=gf,
                                                      panelaction=panelaction)

    def getValues(self):
        '''
        Obtain values selected
        This will be the list of key values for all the checkbox in the tree
        '''
        return self.checkboxtree.getSelection()


class TextMessageWizardPanel(WizardPanel):
    '''
    This class extends the base WizardPanel and presents
    a Textbox.
             +-------------------------------+
             |     some title                |
             +-------------------------------+
             | Here are some instructions..  |
             |                               |
             |                               |
             |                               |
             | ...                           |
             |                               |
             | <back>   <next>   <cancel>    |
             +-------------------------------+
    '''
    def __init__(self, screen, name, paneltitle, msg, buttonbar,
                 position, panelaction):
        '''
        Init method
        :param screen: SnackScreen instance
        :param name: Name of the panel
        :param paneltitle: Panel title displayed to user
        :param msg: Message displayed to user as instructions
        :param buttonbar: Button bar to be added to the panel
        :param position: Panel position in wizard
        :param panelaction: Function to called when user transition out
        '''
        gf = GridForm(screen, paneltitle, 1, 2)
        tb = Textbox(WizardPanel.WIDTH, 10, msg,  scroll=1, wrap=1)

        # Adding widgets to GridForm. First and last widgets
        # will have padding at bottom and top respectively
        # to provide nice spacing.

        gf.add(tb, 0, 0, padding=(0, 0, 0, 1), anchorTop=0)
        bb = buttonbar
        if buttonbar is None:
            bb = buildButtonBar(screen, position)
        gf.add(bb, 0, 1, padding=(0, 1, 0, 0), growx=1)
        self.buttonBar = bb
        super(TextMessageWizardPanel, self).__init__(screen=screen,
                                                     name=name, gridform=gf,
                                                     panelaction=panelaction)


class ExpandedCheckboxTreeWizardPanel(WizardPanel):
    '''
    This class extends the base WizardPanel and simulates
    an expanded CheckboxTree as widget in its gridform.
             +-------------------------------+
             |     some title                |
             +-------------------------------+
             | Here are some instructions..  |
             |                               |
             | + Parent node                 |
             |   [ ] Child node              |
             | ...                           |
             |                               |
             | <back>   <next>   <cancel>    |
             +-------------------------------+
    There can only be one root node, and the tree depth is 1, i.e.
    only 1 list of child node under the root node.
    '''
    def __init__(self, screen, name, paneltitle, msg, checkboxlist,
                 position, panelaction):
        '''
        Init method
        :param screen: SnackScreen instance
        :param name: Name of the panel used as the name of the root node.
        :param paneltitle: Panel title displayed to user
        :param msg: Message displayed to user as instructions
        :param checkboxlist: List of tuple of 2 entries: text, value
               where text is the text to be displayed by the radio button,
               value is the value associated with the button used when
               returning current selection
        :param buttonbar: Button bar to be added to the panel
        :param position: Panel position in wizard
        :param panelaction: Function to called when user transition out
        '''
        height = len(checkboxlist)
        gf = GridForm(screen, paneltitle, 1, height+4)
        tb = Textbox(WizardPanel.WIDTH, 1, msg, scroll=0, wrap=1)
        rootnode = Textbox(WizardPanel.WIDTH, 1, '- '+name, scroll=0)
        self.checkboxlist = []
        count = 0
        g = Grid(2, height)
        for item in checkboxlist:
            if type(item) == tuple:
                (text, val) = item
                cb = Checkbox(text, isOn=0)
                g.setField(cb, 1, count, padding=(2, 0, 1, 0), anchorLeft=1)
                self.checkboxlist.append((cb, val))
                count = count + 1
            else:
                raise MissingTupleException(_("Invalid tuple passed to "
                                              "ExpandedCheckboxTree"
                                              "WizardPanel"))

        gf.add(tb, 0, 0, padding=(0, 0, 0, 1), anchorTop=0)
        gf.add(rootnode, 0, 1, anchorLeft=1, anchorTop=0)
        gf.add(g, 0, 2, anchorLeft=1, anchorTop=0)
        bb = buildButtonBar(screen, position)
        count = count + 1
        gf.add(bb, 0, count, padding=(0, 1, 0, 0), growx=1)
        self.buttonBar = bb
        self.bgrid = g
        super(ExpandedCheckboxTreeWizardPanel,
              self).__init__(screen=screen, name=name, gridform=gf,
                             panelaction=panelaction)

    def getValues(self):
        '''
        Obtain values selected
        This will be the list of key values for all the checkbox in the tree
        '''
        selectedList = []
        for item in self.checkboxlist:
            (cb, val) = item
            if cb.selected():
                selectedList.append(val)
        return selectedList


class AcknowledgementWizardPanel(WizardPanel):
    '''
    This class extends the base WizardPanel and presents
    a Textbox.
             +-------------------------------+
             |           Warning             |
             +-------------------------------+
             | Here are is what will happen..|
             |                               |
             |                               |
             |                               |
             | ...                           |
             |                               |
             |        <Yes>   <No>           |
             +-------------------------------+
    '''

    def __init__(self, screen, name, msg, panelaction):
        '''
        Init method
        :param screen: SnackScreen instance
        :param name: Name of the panel
        :param msg: Message displayed to user as Warning Message
        :param panelaction: Function to called when user transition out
        '''
        gf = GridForm(screen, name, 1, 2)
        # Textbox with message

        tb = TextboxReflowed(WizardPanel.WIDTH, msg)

        # Adding widgets to GridForm. First and last widgets
        # will have padding at bottom and top respectively
        # to provide nice spacing.

        gf.add(tb, 0, 0, padding=(0, 0, 0, 1), anchorTop=0)
        # Bottom buttons

        bb = ButtonBar(screen, ((_("Yes"), WizardPanel.NEXT),
                                (_("No"), WizardPanel.CANCEL)), compact=1)
        gf.add(bb, 0, 1, padding=(0, 1, 0, 0), growx=1)
        self.buttonBar = bb
        super(AcknowledgementWizardPanel, self).__init__(
            screen=screen, name=name, gridform=gf, panelaction=panelaction)


class RadioButtonWizardAdvancePanel(WizardPanel):
    '''
    This class extends the base WizardPanel and uses
    one or more RadioButton as widget in its gridform.
             +-------------------------------------------+
             |          some title                       |
             +-------------------------------------------+
             | Here are some instructions..              |
             |                                           |
             |   Display 1     Display2                  |
             |                                           |
             |     Title1 Title2  Title3                 |
             | --- -----  -----   -----------------------|
             | [ ] Text1  Text2   Text3                  |
             | [ ] Text1  Text2   Text3                  |
             | [ ] Text1  Text2   Text3                  |
             | ...                                       |
             |  Pages                                    |
             | ------------------------------------------|
             |                                           |
             | <view more>  <back>   <next>   <cancel>   |
             +-------------------------------------------+
    '''
    def __init__(self, screen, name, paneltitle, msg, displaylist,
                 titlelist, radiobuttonlist, page, bartype, position,
                 entry_editable, panelaction):
        '''
        Init method
        :param screen: SnackScreen instance
        :param name: Name of the panel
        :param paneltitle: Panel title displayed to user
        :param msg: Message displayed to user as instructions
        :param page: The number of the page
        :param titlelist: A tuple of 3 entries: title1, title2,
               and title3.
        :param radiobuttonlist: List of tuple of 5 entries: text1, text2,
               text3, value, and selected. Text1, text2 and text3 is the
               text to be displayed by the radio button, value is the
               value associated with the button used when returning
               current selection, and selected is the indication of
               whether the button should be selected.
        :param displaylist: A tuple of 1 or 3 entries. The tuple will
            have 1 entry when displaying SDN port selection and 3 entries
            when displaying VIOS(SEA) port selection.
        :param bartype: The type of panel to display: first, middle,
               or last.
        :param position: Panel position in wizard
        :param entry_editable: Boolean to indicate entry is editable
        :param panelaction: Function to called when user transition out
        '''
        self.buttonlist = []
        self.emptyfield = "          "

        gf = GridForm(screen, paneltitle, 1, 5)
        tb = TextboxReflowed(WizardPanel.WIDTH, msg)

        # Display Grid
        displayGrid = Grid(4, 1)
        if type(displaylist) != tuple:
            raise MissingTupleException(_("Invalid display list"
                                          "tuple passed to "
                                          "RadioButtonWizardAdvancePanel"))
        # Display Labels
        displayGrid.setField(Label(str(displaylist[0])), 0, 0,
                             padding=(0, 0, 0, 0), anchorLeft=1)
        displayGrid.setField(Label(self.emptyfield), 1, 0,
                             padding=(0, 0, 0, 0), anchorLeft=1)
        if (len(displaylist) > 1):
            displayGrid.setField(Label(displaylist[1]), 2, 0,
                                 padding=(0, 0, 0, 0), anchorLeft=1)
            self.textEntry = Entry(8, text=str(displaylist[2]))
            if not entry_editable:
                self.textEntry.setFlags(FLAG_DISABLED, FLAGS_SET)
            displayGrid.setField(self.textEntry, 3, 0,
                                 padding=(0, 0, 0, 0), anchorLeft=1)

        titleGrid = Grid(4, 2 + len(radiobuttonlist))

        if type(titlelist) != tuple:
            raise MissingTupleException(_("Invalid title list"
                                          "tuple passed to "
                                          "RadioButtonWizardAdvancePanel"))

        # Title Labels
        titleGrid.setField(Label(RadioButtonWizardAdvancePanel.getspace(3)),
                           0, 0, padding=(0, 0, 0, 0), anchorLeft=1)
        titleGrid.setField(Label(titlelist[0]), 1, 0, padding=(1, 0, 0, 0),
                           anchorLeft=1)
        titleGrid.setField(Label(titlelist[1]), 2, 0, padding=(1, 0, 0, 0),
                           anchorLeft=1)
        titleGrid.setField(Label(titlelist[2]), 3, 0, padding=(1, 0, 0, 0),
                           anchorLeft=1)

        titleGrid.setField(Label(RadioButtonWizardAdvancePanel.getdash(3)),
                           0, 1, padding=(0, 0, 0, 0), anchorLeft=1)
        titleGrid.setField(Label(RadioButtonWizardAdvancePanel.getdash(4)),
                           1, 1, padding=(1, 0, 0, 0), anchorLeft=1)
        titleGrid.setField(Label(RadioButtonWizardAdvancePanel.getdash(26)),
                           2, 1, padding=(1, 0, 0, 0), anchorLeft=1)
        titleGrid.setField(Label(RadioButtonWizardAdvancePanel.getdash(33)),
                           3, 1, padding=(1, 0, 0, 0), anchorLeft=1)

        # Iterate through the radio button list
        x = 2
        for item in radiobuttonlist:
            if type(item) == tuple:
                (t1, t2, t3, val, sel) = item
                self.nextbutton = Checkbox("", sel)
                self.buttonlist.append((self.nextbutton, val))
                # display button

                titleGrid.setField(
                    self.nextbutton, 0, x, padding=(0, 0, 0, 0), anchorLeft=1)
                titleGrid.setField(
                    Label(t1[:4]), 1, x, padding=(1, 0, 0, 0), anchorLeft=1)
                titleGrid.setField(
                    Label(t2[:26]), 2, x, padding=(1, 0, 0, 0), anchorLeft=1)
                titleGrid.setField(
                    Label(t3[:33]), 3, x, padding=(1, 0, 0, 0), anchorLeft=1)
                x += 1
            else:
                raise MissingTupleException(_("Invalid radiobuttonlist "
                                              "tuple passed to "
                                              "RadioButtonWizardAdvancePanel"))

        # Bottom Grid
        # Pages and view more button
        botGrid = Grid(1, 2)
        botGrid.setField(Label(page), 0, 0, padding=(0, 0, 0, 0),
                         anchorLeft=1)

        botGrid.setField(Label(RadioButtonWizardAdvancePanel.getdash(70)),
                         0, 1, padding=(0, 0, 0, 0), anchorLeft=1)

        # Button Bar
        self.buttonBar = RadioButtonWizardAdvancePanel.buildButtonBar(screen,
                                                                      bartype)

        # Add Grids and Bar to the panel
        gf.add(tb, 0, 0, padding=(0, 0, 1, 1), growx=1, anchorLeft=1)
        gf.add(displayGrid, 0, 1, padding=(0, 0, 0, 1), growx=1)
        gf.add(titleGrid, 0, 2, padding=(0, 0, 0, 1), growx=1)
        gf.add(botGrid, 0, 3, padding=(0, 0, 0, 1), growx=1)
        gf.add(self.buttonBar, 0, 4, padding=(0, 0, 0, 0), growx=1)

        super(RadioButtonWizardAdvancePanel, self).__init__(
            screen=screen, name=name, gridform=gf, panelaction=panelaction)

    @staticmethod
    def getdash(value):
        return "-" * value

    @staticmethod
    def getspace(value):
        return " " * value

    def getValues(self):
        '''
        Obtain the list of values selected
        This will be the key associated with the selected radio button.
        '''
        return [value for (b, value) in self.buttonlist if b.selected()]

    def getTextValue(self):
        '''
        Obtain the value in the text entry field
        '''
        return self.textEntry.value()

    @staticmethod
    def buildButtonBar(screen, bartype):
        # Build the button bar accordingly base on bartype
        if bartype == "firstonly":
            bb = ButtonBar(screen, ((_("Back"), WizardPanel.BACK),
                                    (_("Next"), WizardPanel.NEXT),
                                    (_("Cancel"), WizardPanel.CANCEL)),
                           compact=1)
        elif bartype == "first":
            bb = ButtonBar(screen, ((_("View More"), WizardPanel.MORE),
                                    (_("Back"), WizardPanel.BACK),
                                    (_("Next"), WizardPanel.NEXT),
                                    (_("Cancel"), WizardPanel.CANCEL)),
                           compact=1)
        elif bartype == "middle":
            bb = ButtonBar(screen, ((_("View Previous"), WizardPanel.PREV),
                                    (_("View More"), WizardPanel.MORE),
                                    (_("Back"), WizardPanel.BACK),
                                    (_("Next"), WizardPanel.NEXT),
                                    (_("Cancel"), WizardPanel.CANCEL)),
                           compact=1)
        else:
            bb = ButtonBar(screen, ((_("View Previous"), WizardPanel.PREV),
                                    (_("Back"), WizardPanel.BACK),
                                    (_("Next"), WizardPanel.NEXT),
                                    (_("Cancel"), WizardPanel.CANCEL)),
                           compact=1)
        return bb
