# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

from snack import ButtonChoiceWindow
from gettext import gettext as _

'''
Some miscellaneous exceptions
'''


class EmptyListException(Exception):

    def __init__(self):
        self.msg = _("An empty list is given.")

    def __str__(self):
        return self.msg


class InvalidPanelException(Exception):
    def __init__(self):
        self.msg = _("Non WizardPanel object detected.")

    def __str__(self):
        return self.msg


class WizardPanel(object):
    '''
    This class is the base class that represents a panel used
    in the PVM wizard.
    In general it contains a GridForm that will have the following
    rendering:
    +--------------------------------+
    |         title                  |
    +--------------------------------+
    | Some message with instructions |
    |                                |
    | <widget 1>                     |
    |     :                          |
    | <widget n>                     |
    |                                |
    | <button bar back, next, cancel |
    |                                |
    +--------------------------------+

    All panels that wants to be part of the wizard
    must extend this class.
    Methods:
       action: Execution of some code when all data has been validated
       getPanelData: Get all current data on the panel
       getValues: Get values entered by users
       next: Transition to next panel, when user click Next or Back
       run: This method calls the GridForm run to show the panel
       validate: Validate user provided data on the panel
    '''

    '''
    Constants definition for use to with button navigation in wizard
    '''
    NEXT = 1
    FINISH = 1
    AGREE = 1
    BACK = -1
    CANCEL = 0
    # EDIT is a special state to allow control of launching sub panel(s)
    # while a panel is displayed.
    EDIT = 2
    # MORE and PREV are also special state used in iteration over panels
    # when keeping the main wizard flow intact
    PREV = 3
    MORE = 4
    '''
    Constants definition for use to track status of wizard.
    '''
    INIT = 1
    ERROR = -2
    RUNNING = 2
    EXIT = 3
    FINISH = 4
    '''
    Constants definition to indicate panel position in the wizard
    '''
    FIRST = 0
    LAST = -1
    MIDDLE = 1

    '''
    Panel width
    '''
    WIDTH = 68

    def __init__(self, screen, name, gridform, panelaction):
        '''
        For result set to keyword None for now
        '''
        self.result = None
        self.screen = screen
        self.panelaction = panelaction
        self.name = name
        self.gridForm = gridform

    def action(self, *args):
        '''
        Perform some action. This method is called prior to transitioning
        to the next panel. Base implementation just returns 0 if no panelaction
        call is registered.
        I chose to have panelaction saved as a function to be called because I
        think the complexity of running an action may make the code in
        the extended class too complicated. Class can just override
        the action function and not bother with registering an actionpanel
        function.
        '''
        if callable(self.panelaction):
            self.panelaction(*args)

    def getPanelData(self):
        '''
        Obtain all current values from the panel
        '''
        return None

    def getValues(self):
        '''
        Obtain values from user selection. Base implementation returns None.
        '''
        return None

    def next(self):
        '''
        Performs step to determine the next action
        value returns is one of:
        0 = Cancel wizard
        1 = Back to previous panel
        2 = Stay in panel
        -1 = Next panel
        '''
        if self.buttonBar is None:
            return WizardPanel.CANCEL
        else:
            val = self.buttonBar.buttonPressed(self.result)
            if not self.result or val is None:
                return WizardPanel.NEXT
            return val

    def run(self):
        '''
        Display the panel
        '''
        self.result = self.gridForm.run()
        self.screen.popWindow(True)

    @staticmethod
    def showError(screen, msg):
        '''
        Show an error message
        '''
        ButtonChoiceWindow(screen, _("Error"),
                           msg, buttons=[_("OK")])

    def validate(self):
        '''
        Validates data. Mainly needed if panel have user entries.
        A non-zero returns code indicates error with data entered.
        '''
        pass
